import java.io.IOException;
import java.io.Serializable;
import java.net.URL;
import java.util.List;
import javax.transaction.RollbackException;
import javax.transaction.TransactionManager;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.infinispan.Cache;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.remoting.transport.Address;

public class ThreeNodesDeadlocks {
	private static final Logger logger = Logger.getLogger(ThreeNodesDeadlocks.class);

	public static final String COUNTER_KEY="counter_key";
	public static final String CACHE_NAME="CACHE_NAME";
	
	Cache<String, Serializable> theCache =null;
	EmbeddedCacheManager manager=null;
	
	public ThreeNodesDeadlocks() {
		try {
			manager = new DefaultCacheManager("AgentManager_InfiniSpan_Config_Generic.xml");
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
		logger.info("is deadlock detection activated="+manager.getDefaultConfiguration().isEnableDeadlockDetection());
		theCache = manager.getCache(CACHE_NAME);
		theCache.put(COUNTER_KEY, new Integer(0));
	}
	
	public void waitForAllNodesStart(){
		int nbMembers=0;
		do {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			List<Address> theMemberList=manager.getMembers();
			nbMembers=theMemberList.size();
		}while (nbMembers!=3);
		logger.info("All nodes have started: waiting 3 secondes to initialize in order to avoid ISPN-595 issue case...");
		try {
			Thread.sleep(3000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		logger.info("TransportNodeName2="+manager.getAddress());
	}
	
	public void launchOperation(){
		TransactionManager tm = theCache.getAdvancedCache().getTransactionManager();
		try {
			tm.begin();
			logger.info("Before LOCK...");
			theCache.getAdvancedCache().lock(COUNTER_KEY);
			logger.info("After LOCK...");
			Integer counter=(Integer)theCache.get(COUNTER_KEY);
			//counter=counter.intValue()+1;
			theCache.put(COUNTER_KEY, counter+1);
			tm.commit();
		}  catch (RollbackException e) {
			logger.warn("Rollback exception.");
			e.printStackTrace();
		} catch (Exception e) {
			logger.warn("General exception.");
			System.exit(1);
		} 
	}
	
	
	
	public void testResult(){
		logger.info("nbAgents="+(Integer)theCache.get(COUNTER_KEY));
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// URL theFileURL=ThreeNodesDeadlocks.class.getClassLoader().getResource("log4j_Generic.properties");
		// System.out.println("fileURL="+theFileURL);
		// PropertyConfigurator.configure(theFileURL);

		
		
		ThreeNodesDeadlocks launcher=new ThreeNodesDeadlocks();
		// We wait all nodes to start to avoid the issue ISPN-595.
		launcher.waitForAllNodesStart();
		int limit=100;
		int i=0;
		for(i=0;i<limit;i++){
			launcher.launchOperation();
		}
		launcher.testResult();
		logger.info("############# allEnded.");
	}

}
